@@ -31,6 +31,7 @@ class IScopedLock;
 class IDatabaseFactory;
 class IPipeThrottler;
 class IPipeThrottlerUpdater;
+class IWebSocket;
 
 struct SPostfile
 {
@@ -80,6 +81,9 @@ public:
 	virtual void setActionContext(std::string context)=0;
 	virtual void resetActionContext(void)=0;
 
+	virtual void addWebSocket(IWebSocket* websocket) = 0;
+	virtual THREAD_ID ExecuteWebSocket(const std::string& name, str_map& GET, str_map& PARAMS, IPipe* pipe, const std::string& endpoint_name) = 0;
+
 	virtual int64 getTimeSeconds(void)=0;
 	virtual int64 getTimeMS(void)=0;
 
@@ -148,6 +152,7 @@ public:
 	virtual void DisconnectStream(IPipe *pipe)=0;
 	virtual std::string LookupHostname(const std::string& pIp)=0;
 
+	virtual std::vector<std::string> getStreamPipeList() = 0;
 	virtual bool RegisterPluginPerThreadModel(IPluginMgr *pPluginMgr, std::string pName)=0;
 	virtual bool RegisterPluginThreadsafeModel(IPluginMgr *pPluginMgr, std::string pName)=0;
 
