@@ -87,6 +87,9 @@ public:
 	virtual void setActionContext(std::string context);
 	virtual void resetActionContext(void);
 
+	virtual void addWebSocket(IWebSocket* websocket);
+	virtual THREAD_ID ExecuteWebSocket(const std::string& name, str_map& GET, str_map& PARAMS, IPipe* pipe, const std::string& endpoint_name);
+
 	virtual int64 getTimeSeconds(void);
 	virtual int64 getTimeMS(void);
 
@@ -143,6 +146,7 @@ public:
 	virtual void DisconnectStream(IPipe *pipe);
 	virtual std::string LookupHostname(const std::string& pIp);
 
+	virtual std::vector<std::string> getStreamPipeList();
 	virtual bool RegisterPluginPerThreadModel(IPluginMgr *pPluginMgr, std::string pName);
 	virtual bool RegisterPluginThreadsafeModel(IPluginMgr *pPluginMgr, std::string pName);
 
@@ -224,7 +228,7 @@ private:
 
 	void rotateLogfile();
 
-	IPipe* ConnectStream(const SLookupBlockingResult& lookup_result, unsigned short pPort, unsigned int pTimeoutms);
+	IPipe* ConnectStream(const std::string& connect_str, const SLookupBlockingResult& lookup_result, unsigned short pPort, unsigned int pTimeoutms);
 
 
 	int loglevel;
@@ -233,6 +237,7 @@ private:
 
 	IMutex* log_mutex;
 	IMutex* action_mutex;
+	IMutex* web_socket_mutex;
 	IMutex* requests_mutex;
 	IMutex* outputs_mutex;
 	IMutex* db_mutex;
@@ -249,6 +254,8 @@ private:
 
 	std::map< std::string, std::map<std::string, IAction*> > actions;
 
+	std::map<std::string, IWebSocket*> web_sockets;
+
 	std::map<std::string, UNLOADACTIONS> unload_functs;
 	std::vector<HMODULE> unload_handles;
 
