@@ -32,7 +32,12 @@ private:
 	bool TcpStep(void);
 	void DelClientThreads(void);
 
+	bool startIpv4(_u16 tcpport);
+
+	bool startIpv6(_u16 tcpport);
+
 	SOCKET mSocket;
+	SOCKET mSocketv6;
 
 	std::vector<CClientThread*> clientthreads;
 	CUDPThread *udpthread;
