@@ -11,6 +11,7 @@
 #include <memory.h>
 #include <cstring>
 #include <stdlib.h>
+#include <assert.h>
 
 #ifndef _S_IFIFO
 #define	_S_IFIFO 0x1000
@@ -49,7 +50,7 @@ namespace
 
 	std::string extract_string(const std::string& header, size_t off, size_t size)
 	{
-		for (size_t i = off; i < off+size; ++i)
+		for (size_t i = off; i < off + size; ++i)
 		{
 			if (header[i] == 0)
 			{
@@ -59,6 +60,21 @@ namespace
 		return header.substr(off, size);
 	}
 
+	std::string extract_string_for_number(const std::string& header, size_t off, size_t size)
+	{
+		if (header[off] > 0)
+		{
+			for (size_t i = off; i < off + size; ++i)
+			{
+				if (header[i] == 0)
+				{
+					return header.substr(off, i - off);
+				}
+			}
+		}
+		return header.substr(off, size);
+	}
+
 	std::string extract_digits(const std::string& val)
 	{
 		std::string ret;
@@ -74,7 +90,7 @@ namespace
 
 	int64 extract_number(const std::string& header, size_t off, size_t size)
 	{
-		std::string val = extract_string(header, off, size);
+		std::string val = extract_string_for_number(header, off, size);
 
 		if (!val.empty() && val[0] < 0)
 		{
@@ -84,12 +100,7 @@ namespace
 
 			for (size_t i = 1; i < val.size(); ++i)
 			{
-				f = val[i];
-				if (f == 0)
-				{
-					break;
-				}
-				res = (res << 8) | f;
+				res = (res << 8) | val[i];
 			}
 
 			if (neg)
@@ -103,7 +114,7 @@ namespace
 		return strtoll(extract_digits(val).c_str(), NULL, 8);
 	}
 
-	bool check_header_checksum(const std::string& header)
+	bool check_header_checksum(const std::string& header, std::string& errmsg)
 	{
 		int64 checksum = extract_number(header, 148, 8);
 
@@ -125,6 +136,7 @@ namespace
 
 		if (checksum1 != checksum && checksum2 != checksum)
 		{
+			errmsg = "TAR checksum wrong. Checksum=" + convert(checksum) + " checksum1=" + convert(checksum1) + " checksum2=" + convert(checksum2);
 			return false;
 		}
 		else
@@ -136,14 +148,14 @@ namespace
 
 PipeFileTar::PipeFileTar(const std::string & pCmd, int backupnum, int64 fn_random, std::string output_fn, const std::string& server_token, const std::string& identity)
 	: pipe_file(new PipeFileStore(new PipeFile(pCmd))), file_offset(0), mutex(Server->createMutex()), backupnum(backupnum), has_next(false), output_fn(output_fn), fn_random(fn_random),
-	server_token(server_token), identity(identity)
+	server_token(server_token), identity(identity), hash_pos(0)
 {
 	sha_def_init(&sha_ctx);
 }
 
 PipeFileTar::PipeFileTar(PipeFileStore* pipe_file, const STarFile& tar_file, int64 file_offset, int backupnum, int64 fn_random, std::string output_fn, const std::string& server_token, const std::string& identity)
 	: pipe_file(pipe_file), tar_file(tar_file), file_offset(file_offset), mutex(Server->createMutex()), backupnum(backupnum), has_next(false), output_fn(output_fn), fn_random(fn_random),
-	server_token(server_token), identity(identity)
+	server_token(server_token), identity(identity), hash_pos(0)
 {
 	sha_def_init(&sha_ctx);
 }
@@ -170,7 +182,8 @@ std::string PipeFileTar::Read(_u32 tr, bool * has_error)
 
 	lock.relock(mutex.get());
 
-	sha_def_update(&sha_ctx, reinterpret_cast<const unsigned char*>(ret.data()), ret.size());
+	hashReadData(pos, ret.data(), ret.size());
+	assert(pos + ret.size() <= hash_pos);
 
 	tar_file.pos += ret.size();
 	return ret;
@@ -178,6 +191,7 @@ std::string PipeFileTar::Read(_u32 tr, bool * has_error)
 
 std::string PipeFileTar::Read(int64 spos, _u32 tr, bool * has_error)
 {
+	assert(spos <= hash_pos);
 	IScopedLock lock(mutex.get());
 
 	if (!tar_file.available)
@@ -194,7 +208,8 @@ std::string PipeFileTar::Read(int64 spos, _u32 tr, bool * has_error)
 	std::string ret = pipe_file->pipe_file->Read(pf_offset, max_read, has_error);
 
 	lock.relock(mutex.get());
-	sha_def_update(&sha_ctx, reinterpret_cast<const unsigned char*>(ret.data()), ret.size());
+	hashReadData(spos, ret.data(), ret.size());
+	assert(spos + ret.size() <= hash_pos);
 
 	return ret;
 }
@@ -211,7 +226,8 @@ _u32 PipeFileTar::Read(char * buffer, _u32 bsize, bool * has_error)
 
 	lock.relock(mutex.get());
 
-	sha_def_update(&sha_ctx, reinterpret_cast<const unsigned char*>(buffer), ret);
+	hashReadData(tar_file.pos, buffer, ret);
+	assert(pos + ret <= hash_pos);
 	
 	tar_file.pos += ret;
 	return ret;
@@ -219,6 +235,7 @@ _u32 PipeFileTar::Read(char * buffer, _u32 bsize, bool * has_error)
 
 _u32 PipeFileTar::Read(int64 spos, char * buffer, _u32 bsize, bool * has_error)
 {
+	assert(spos <= hash_pos);
 	IScopedLock lock(mutex.get());
 
 	if (!tar_file.available)
@@ -234,7 +251,8 @@ _u32 PipeFileTar::Read(int64 spos, char * buffer, _u32 bsize, bool * has_error)
 	_u32 ret = pipe_file->pipe_file->Read(pf_offset, buffer, bsize, has_error);
 
 	lock.relock(mutex.get());
-	sha_def_update(&sha_ctx, reinterpret_cast<const unsigned char*>(buffer), ret);
+	hashReadData(spos, buffer, ret);
+	assert(spos + ret <= hash_pos);
 
 	return ret;
 }
@@ -247,6 +265,7 @@ bool PipeFileTar::Seek(_i64 spos)
 		return false;
 	}
 
+	assert(spos <= hash_pos);
 	tar_file.pos = spos;
 
 	int64 pf_offset = file_offset + spos;
@@ -256,11 +275,11 @@ bool PipeFileTar::Seek(_i64 spos)
 	return pipe_file->pipe_file->Seek(pf_offset);
 }
 
-bool PipeFileTar::switchNext(std::string& fn, bool& is_dir, bool& is_symlink, bool& is_special, std::string& symlink_target, int64& size, bool *has_error)
+bool PipeFileTar::switchNext(std::string& fn, bool& is_dir, bool& is_symlink, bool& is_special, std::string& symlink_target, int64& size, std::string & stderr_ret, bool *has_error)
 {
 	IScopedLock lock(mutex.get());
 
-	bool b= readHeader(has_error);
+	bool b= readHeader(has_error, stderr_ret);
 	if (b)
 	{
 		fn = tar_file.fn;
@@ -280,6 +299,30 @@ _i64 PipeFileTar::Size()
 	return tar_file.size;
 }
 
+void PipeFileTar::hashReadData(int64 spos, const char * buffer, _u32 bsize)
+{
+	_u32 off = 0;
+	if (spos < hash_pos)
+	{
+		off = hash_pos - spos;
+		if (off >= bsize)
+		{
+			return;
+		}
+		Server->Log("Hash tar file off!=0", LL_DEBUG);
+	}
+	if (spos+off == hash_pos)
+	{
+		Server->Log("Hash tar file spos=" + convert(spos) + " len=" + convert(bsize) +" off="+convert(off), LL_DEBUG);
+		sha_def_update(&sha_ctx, reinterpret_cast<const unsigned char*>(buffer+off), bsize-off);
+		hash_pos += bsize-off;
+	}
+	else
+	{
+		Server->Log("Don't hash tar file spos=" + convert(spos) + " len=" + convert(bsize), LL_DEBUG);
+	}
+}
+
 std::string PipeFileTar::buildCurrMetadata()
 {
 	CWData data;
@@ -369,7 +412,7 @@ std::string PipeFileTar::buildCurrMetadata()
 	return std::string(data.getDataPtr(), data.getDataSize());
 }
 
-bool PipeFileTar::readHeader(bool* has_error)
+bool PipeFileTar::readHeader(bool* has_error, std::string & stderr_ret)
 {
 	int64 header_pos = file_offset + roundUp(tar_file.size, 512);
 	int64 cpos = pipe_file->pipe_file->getPos();
@@ -381,6 +424,7 @@ bool PipeFileTar::readHeader(bool* has_error)
 
 	if (header.size() != 512)
 	{
+		addErrMsg("Error reading tar header (1). Unexpected length " + convert(header.size()), stderr_ret);
 		if(has_error!=NULL) *has_error = true;
 		return false;
 	}
@@ -391,18 +435,23 @@ bool PipeFileTar::readHeader(bool* has_error)
 
 		if (header.size() != 512)
 		{
+			addErrMsg("Error reading tar header (2). Unexpected length " + convert(header.size()), stderr_ret);
 			if (has_error != NULL) *has_error = true;
 			return false;
 		}
 
 		if (is_zeroes(header))
 		{
+			Server->Log("Tar file EOF", LL_DEBUG);
 			return false;
 		}
 	}
 
-	if (!check_header_checksum(header))
+	std::string errmsg;
+	if (!check_header_checksum(header, errmsg))
 	{
+		addErrMsg(errmsg, stderr_ret);
+		addErrMsg("Current tar fn: "+extract_string(header, 0, 100), stderr_ret);
 		if (has_error != NULL) *has_error = true;
 		return false;
 	}
@@ -487,6 +536,24 @@ bool PipeFileTar::readHeader(bool* has_error)
 	return true;
 }
 
+void PipeFileTar::addErrMsg(const std::string& msg, std::string & stderr_ret)
+{
+	int64 timems = little_endian(Server->getTimeMS());
+	unsigned int le_size = little_endian(static_cast<unsigned int>(msg.size()+1));
+
+	stderr_ret.append(1, 0);
+
+	stderr_ret.append(reinterpret_cast<const char*>(&timems),
+		reinterpret_cast<const char*>(&timems) + sizeof(timems));
+
+	stderr_ret.append(reinterpret_cast<const char*>(&le_size),
+		reinterpret_cast<const char*>(&le_size) + sizeof(le_size));
+
+	stderr_ret.append(msg+"\n");
+
+	Server->Log(msg, LL_ERROR);
+}
+
 _u32 PipeFileTar::Write(const std::string & tw, bool * has_error)
 {
 	return 0;
@@ -554,6 +621,8 @@ std::string PipeFileTar::getStdErr()
 	unsigned char dig[SHA_DEF_DIGEST_SIZE];
 	sha_def_final(&sha_ctx, dig);
 
+	Server->Log("Hash tar file final (" + tar_file.fn + ") pos="+convert(tar_file.pos)+" hash_pos="+convert(hash_pos)+" size="+convert(tar_file.size)+" h="+base64_encode(dig, SHA_DEF_DIGEST_SIZE), LL_DEBUG);
+
 	stderr_ret.append(1, 1);
 
 	stderr_ret.append(reinterpret_cast<const char*>(dig),
@@ -572,7 +641,7 @@ std::string PipeFileTar::getStdErr()
 
 	while (true)
 	{
-		if(switchNext(fn, is_dir, is_symlink, is_special, symlink_target, size))
+		if(switchNext(fn, is_dir, is_symlink, is_special, symlink_target, size, stderr_ret))
 		{
 			if (!fn.empty() && fn[fn.size() - 1] == '/')
 			{
@@ -704,6 +773,7 @@ std::string PipeFileTar::getStdErr()
 
 bool PipeFileTar::getExitCode(int & exit_code)
 {
+	IScopedLock lock(mutex.get());
 	if (has_next)
 	{
 		exit_code = 0;
@@ -711,14 +781,17 @@ bool PipeFileTar::getExitCode(int & exit_code)
 	}
 	else
 	{
+		lock.relock(NULL);
 		return pipe_file->pipe_file->getExitCode(exit_code);
 	}
 }
 
 void PipeFileTar::forceExitWait()
 {
+	IScopedLock lock(mutex.get());
 	if (!has_next)
 	{
+		lock.relock(NULL);
 		pipe_file->pipe_file->forceExitWait();
 	}
 }
@@ -743,3 +816,16 @@ int64 PipeFileTar::getPos()
 	IScopedLock lock(mutex.get());
 	return tar_file.pos;
 }
+
+bool PipeFileTar::waitForStderr(int64 timeoutms)
+{
+	IScopedLock lock(mutex.get());
+	if (has_next)
+	{
+		return true;
+	}
+	else
+	{
+		return pipe_file->pipe_file->waitForStderr(timeoutms);
+	}
+}
