@@ -363,6 +363,11 @@ void FSNTFS::logFileChanges(std::string volpath, int64 min_size, char * fc_bitma
 {
 }
 
+std::string FSNTFS::getType()
+{
+	return "ntfs";
+}
+
 bool FSNTFS::checkMFTMirror(unsigned int mftrecordsize, Runlist &mftrunlist, NTFSFileRecord &mft, bool fix)
 {
 	unsigned int mirr_vcn=(1*mftrecordsize)/clustersize;
