@@ -89,12 +89,12 @@ bool setSockP(SOCKET sock)
         return true;
 }    
 
-FileClient::FileClient(bool enable_find_servers, std::string identity, int protocol_version, bool internet_connection,
+FileClient::FileClient(bool enable_find_servers, std::string identity, int protocol_version, bool add_request_checksums,
 	FileClient::ReconnectionCallback *reconnection_callback, FileClient::NoFreeSpaceCallback *nofreespace_callback)
 	: tcpsock(NULL), starttime(0), connect_starttime(0), socket_open(false), connected(false),
 	local_ip(),
 	max_version(), server_addr(), connection_id(), 
-	protocol_version(protocol_version), internet_connection(internet_connection),
+	protocol_version(protocol_version), 
 	transferred_bytes(0), reconnection_callback(reconnection_callback),
 	nofreespace_callback(nofreespace_callback), reconnection_timeout(300000), retryBindToNewInterfaces(true),
 	identity(identity), received_data_bytes(0), queue_callback(NULL), dl_off(0),
@@ -110,7 +110,7 @@ FileClient::FileClient(bool enable_find_servers, std::string identity, int proto
 	}
 
 	socket_open=false;
-	stack.setAddChecksum(internet_connection);
+	stack.setAddChecksum(add_request_checksums);
 
 	mutex = Server->createMutex();
 }
@@ -582,7 +582,7 @@ _u32 FileClient::getLocalIP(void)
         return local_ip;
 }
 
-_u32 FileClient::GetServers(bool start, const std::vector<SAddrHint> &addr_hints)
+_u32 FileClient::GetServers(bool start, const std::vector<SAddrHint> &addr_hints, GetServersErrors& errors)
 {
         if(start)
         {
@@ -610,7 +610,14 @@ _u32 FileClient::GetServers(bool start, const std::vector<SAddrHint> &addr_hints
 					int rc = sendto(udpsocks[i].udpsock, &ch, 1, 0, (sockaddr*)&addr_udp, sizeof(addr_udp));
 					if (rc == -1)
 					{
-						Server->Log("Sending broadcast failed!", LL_ERROR);
+						if(!errors.broadcast_error_ipv4)
+							Server->Log("Sending broadcast failed!", LL_ERROR);
+
+						errors.broadcast_error_ipv4 = true;
+					}
+					else
+					{
+						errors.broadcast_error_ipv4 = false;
 					}
 				}
 				else
@@ -620,14 +627,24 @@ _u32 FileClient::GetServers(bool start, const std::vector<SAddrHint> &addr_hints
 					addr_udp.sin6_port = htons(UDP_PORT);
 					if (inet_pton(AF_INET6, multicast_group, &addr_udp.sin6_addr) != 1)
 					{
-						Server->Log("inet_pton failed", LL_ERROR);
+						if(!errors.broadcast_error_ipv6)
+							Server->Log("inet_pton failed", LL_ERROR);
+
+						errors.broadcast_error_ipv6 = true;
 					}
 
 					char ch = ID_PING;
 					int rc = sendto(udpsocks[i].udpsock, &ch, 1, 0, (sockaddr*)&addr_udp, sizeof(addr_udp));
 					if (rc == -1)
 					{
-						Server->Log("Sending broadcast failed! (ipv6)", LL_ERROR);
+						if (!errors.broadcast_error_ipv6)
+							Server->Log("Sending broadcast failed! (ipv6)", LL_ERROR);
+
+						errors.broadcast_error_ipv6 = true;
+					}
+					else
+					{
+						errors.broadcast_error_ipv6 = false;
 					}
 				}
 			}
@@ -1116,6 +1133,12 @@ bool FileClient::Reconnect(void)
 			{
 				rc += dl_off;
 			}
+
+			if (firstpacket && dl_off + rc > 0 && dl_off + rc < 10 && buf[0] == ID_ERR)
+			{
+				Server->Log("Received ID_ERR from server rc=" + convert(rc)+". Reconnecting...", LL_WARNING);
+				rc = 0;
+			}
 		}
 		else
 		{
@@ -2005,6 +2028,11 @@ void FileClient::setReconnectTries(int tries)
 	reconnect_tries = tries;
 }
 
+void FileClient::setAddChecksum(bool add_request_checksums)
+{
+	stack.setAddChecksum(add_request_checksums);
+}
+
 void FileClient::setProgressLogCallback( ProgressLogCallback* cb )
 {
 	progress_log_callback = cb;
