@@ -54,7 +54,7 @@ FileBackup::FileBackup( ClientMain* client_main, int clientid, std::string clien
 	:  Backup(client_main, clientid, clientname, clientsubname, log_action, true, is_incremental, server_token, details, scheduled),
 	group(group), use_tmpfiles(use_tmpfiles), tmpfile_path(tmpfile_path), use_reflink(use_reflink), use_snapshots(use_snapshots),
 	disk_error(false), with_hashes(false),
-	backupid(-1), hashpipe(NULL), hashpipe_prepare(NULL), bsh(NULL), bsh_prepare(NULL),
+	backupid(-1), hashpipe(NULL), hashpipe_prepare(NULL),
 	bsh_ticket(ILLEGAL_THREADPOOL_TICKET), bsh_prepare_ticket(ILLEGAL_THREADPOOL_TICKET), pingthread(NULL),
 	pingthread_ticket(ILLEGAL_THREADPOOL_TICKET), cdp_path(false), metadata_download_thread_ticket(ILLEGAL_THREADPOOL_TICKET),
 	last_speed_received_bytes(0), speed_set_time(0)
@@ -119,7 +119,7 @@ bool FileBackup::request_filelist_construct(bool full, bool resume, int group,
 	CTCPStack tcpstack(client_main->isOnInternetConnection());
 
 	ServerLogger::Log(logid, clientname+": Connecting for filelist...", LL_DEBUG);
-	IPipe *cc=client_main->getClientCommandConnection(server_settings.get(), 10000);
+	IPipe *cc=client_main->getClientCommandConnection(server_settings.get(), 60000);
 	if(cc==NULL)
 	{
 		ServerLogger::Log(logid, "Connecting to ClientService of \""+clientname+"\" failed - CONNECT error during filelist construction", LL_ERROR);
@@ -407,7 +407,7 @@ bool FileBackup::wait_for_async(const std::string& async_id, int64 timeout_time)
 				&& Server->getTimeMS() - starttime <= timeout_time)
 			{
 				ServerLogger::Log(logid, clientname + ": Connecting for async...", LL_DEBUG);
-				cc.reset(client_main->getClientCommandConnection(server_settings.get(), 10000));
+				cc.reset(client_main->getClientCommandConnection(server_settings.get(), 60000));
 
 				if (ServerStatus::getProcess(clientname, status_id).stop)
 				{
@@ -561,7 +561,7 @@ bool FileBackup::getTokenFile(FileClient &fc, bool hashed_transfer, bool request
 		!access_key.empty() &&
 		access_key != client_access_key )
 	{
-		backup_dao->updateOrInsertSetting(clientid, "client_access_key", access_key);
+		backup_dao->updateOrInsertSetting(clientid, "client_access_key", access_key, "", c_use_value, 0);
 
 		if(!client_access_key.empty())
 		{
@@ -581,36 +581,43 @@ std::string FileBackup::clientlistName(int ref_backupid)
 
 void FileBackup::createHashThreads(bool use_reflink, bool ignore_hash_mismatches)
 {
-	assert(bsh==NULL);
-	assert(bsh_prepare==NULL);
+	assert(bsh.empty());
+	assert(bsh_prepare.empty());
 
 	hashpipe=Server->createMemoryPipe();
 	hashpipe_prepare=Server->createMemoryPipe();
 
-	bsh=new BackupServerHash(hashpipe, clientid, use_snapshots, use_reflink, use_tmpfiles, logid, use_snapshots, max_file_id);
-	bsh_prepare=new BackupServerPrepareHash(hashpipe_prepare, hashpipe, clientid, logid, ignore_hash_mismatches);
-	bsh_ticket = Server->getThreadPool()->execute(bsh, "fbackup write");
-	bsh_prepare_ticket = Server->getThreadPool()->execute(bsh_prepare, "fbackup hash");
+	size_t h_cnt = server_settings->getSettings()->hash_threads;
+
+	for (size_t i = 0; i < h_cnt; ++i)
+	{
+		BackupServerHash* curr_bsh = new BackupServerHash(hashpipe, clientid, use_snapshots, use_reflink, use_tmpfiles, logid, use_snapshots, max_file_id);
+		BackupServerPrepareHash* curr_bsh_prepare = new BackupServerPrepareHash(hashpipe_prepare, hashpipe, clientid, logid, ignore_hash_mismatches);
+		bsh.push_back(curr_bsh);
+		bsh_prepare.push_back(curr_bsh_prepare);
+		bsh_ticket.push_back(Server->getThreadPool()->execute(curr_bsh, "fbackup write" + convert(i)));
+		bsh_prepare_ticket.push_back(Server->getThreadPool()->execute(curr_bsh_prepare, "fbackup hash" + convert(i)));
+	}
 }
 
 
 void FileBackup::destroyHashThreads()
 {
-	if (hashpipe_prepare != NULL)
+	if (!bsh_prepare.empty())
 	{
-		assert(bsh_ticket != ILLEGAL_THREADPOOL_TICKET);
-		assert(bsh_prepare_ticket != ILLEGAL_THREADPOOL_TICKET);
 		hashpipe_prepare->Write("exit");
 		Server->getThreadPool()->waitFor(bsh_ticket);
 		Server->getThreadPool()->waitFor(bsh_prepare_ticket);
+		Server->destroy(hashpipe_prepare);
+		Server->destroy(hashpipe);
 	}
 
-	bsh_ticket=ILLEGAL_THREADPOOL_TICKET;
-	bsh_prepare_ticket=ILLEGAL_THREADPOOL_TICKET;
+	bsh_ticket.clear();
+	bsh_prepare_ticket.clear();
 	hashpipe=NULL;
 	hashpipe_prepare=NULL;
-	bsh=NULL;
-	bsh_prepare=NULL;
+	bsh.clear();
+	bsh_prepare.clear();
 }
 
 _i64 FileBackup::getIncrementalSize(IFile *f, const std::vector<size_t> &diffs, bool& backup_with_components, bool all)
@@ -781,7 +788,7 @@ bool FileBackup::doBackup()
 		return false;
 	}
 
-	if( server_settings->getSettings()->internet_mode_enabled )
+	if( client_main->isOnInternetConnection() )
 	{
 		if( server_settings->getSettings()->internet_incr_file_transfer_mode=="blockhash")
 		{
@@ -794,6 +801,17 @@ bool FileBackup::doBackup()
 		with_hashes=true;
 	}
 
+	ServerBackupDao::CondInt c_with_hashes = backup_dao->getClientWithHashes(clientid);
+	if (c_with_hashes.exists
+		&& c_with_hashes.value != 0)
+	{
+		with_hashes = true;
+	}
+	else if (with_hashes)
+	{
+		backup_dao->updateClientWithHashes(1, clientid);
+	}
+
 	if(!fileindex.get())
 	{
 		fileindex.reset(create_lmdb_files_index());
@@ -1000,6 +1018,39 @@ std::string FileBackup::fixFilenameForOS(std::string fn, std::set<std::string>&
 		}
 		fn.resize(name_max);
 		append_hash = true;
+
+		size_t rm_bytes = 0;
+		//Repair UTF-8
+		for (size_t i = fn.size() - 1; i-- > 0;)
+		{
+			const unsigned char first_mask = 0x80;
+			const unsigned char utf8_start = 0xC0;
+
+			const unsigned char ch = static_cast<unsigned char>(fn[i]);
+
+			if (ch & first_mask)
+			{
+				if (ch & utf8_start == utf8_start)
+				{
+					++rm_bytes;
+					break;
+				}
+				else
+				{
+					++rm_bytes;
+				}
+			}
+			else
+			{
+				//ASCII char
+				break;
+			}
+		}
+
+		if (rm_bytes > 0)
+		{
+			fn.resize(name_max - rm_bytes);
+		}
 	}
 #endif
 
@@ -1208,18 +1259,30 @@ void FileBackup::waitForFileThreads(void)
 	SStatus status=ServerStatus::getStatus(clientname);
 	hashpipe->Write("flush");
 	hashpipe_prepare->Write("flush");
-	_u32 hashqueuesize=(_u32)hashpipe->getNumElements()+(bsh->isWorking()?1:0);
-	_u32 prepare_hashqueuesize=(_u32)hashpipe_prepare->getNumElements()+(bsh_prepare->isWorking()?1:0);
-	while(hashqueuesize>0 || prepare_hashqueuesize>0)
+	
+	size_t hashqueuesize=std::string::npos;
+	size_t prepare_hashqueuesize=0;
+	while(hashqueuesize==std::string::npos || hashqueuesize>0 || prepare_hashqueuesize>0)
 	{
-		ServerStatus::setProcessQueuesize(clientname, status_id, prepare_hashqueuesize, hashqueuesize);
-		Server->wait(1000);
-		hashqueuesize=(_u32)hashpipe->getNumElements()+(bsh->isWorking()?1:0);
-		prepare_hashqueuesize=(_u32)hashpipe_prepare->getNumElements()+(bsh_prepare->isWorking()?1:0);
+		if (hashqueuesize != std::string::npos)
+		{
+			ServerStatus::setProcessQueuesize(clientname, status_id, prepare_hashqueuesize, hashqueuesize);
+			Server->wait(1000);
+		}
+
+		size_t bsh_working = bsh.size() - hashpipe->getNumWaiters();
+		size_t bsh_prepare_working = bsh_prepare.size() - hashpipe_prepare->getNumWaiters();
+		
+		hashqueuesize = hashpipe->getNumElements() + bsh_working;
+		prepare_hashqueuesize = hashpipe_prepare->getNumElements() + bsh_prepare_working;
 	}
 	{
 		Server->wait(10);
-		while(bsh->isWorking()) Server->wait(1000);
+
+		while (hashpipe->getNumWaiters() < bsh.size())
+		{
+			Server->wait(1000);
+		}
 	}	
 
 	ServerStatus::setProcessQueuesize(clientname, status_id, 0, 0);
@@ -1233,7 +1296,7 @@ bool FileBackup::verify_file_backup(IFile *fileentries)
 
 	std::ostringstream log;
 
-	log << "Verification of file backup with id " << backupid << ". Path=" << (backuppath) << " Tree-hashing=" << convert(BackupServer::useTreeHashing()) << std::endl;
+	log << "Verification of file backup with id " << backupid << ". Path=" << (backuppath) << " Tree-hashing=" << convert(BackupServer::useTreeHashing()) << "\r\n";
 
 	unsigned int read;
 	char buffer[4096];
@@ -1314,7 +1377,7 @@ bool FileBackup::verify_file_backup(IFile *fileentries)
 								std::string msg = "No hash for file \"" + (curr_path + os_file_sep() + cf.name) + "\" found. Verification failed.";
 								verify_ok = false;
 								ServerLogger::Log(logid, msg, LL_ERROR);
-								log << msg << std::endl;
+								log << msg << "\r\n";
 							}
 						}
 						else
@@ -1326,7 +1389,7 @@ bool FileBackup::verify_file_backup(IFile *fileentries)
 								std::string msg="Hashes for \""+(curr_path+os_file_sep()+cf.name)+"\" differ (client side hash). Verification failed.";
 								verify_ok=false;
 								ServerLogger::Log(logid, msg, LL_ERROR);
-								log << msg << std::endl;
+								log << msg << "\r\n";
 								save_debug_data(remote_path+"/"+cf.name,
 									base64_encode_dash(getSHADef(curr_path+os_file_sep()+cfn)),
 									shabase64);
@@ -1344,7 +1407,7 @@ bool FileBackup::verify_file_backup(IFile *fileentries)
 							std::string msg="Hashes for \""+(curr_path+os_file_sep()+cf.name)+"\" differ. Verification failed.";
 							verify_ok=false;
 							ServerLogger::Log(logid, msg, LL_ERROR);
-							log << msg << std::endl;
+							log << msg << "\r\n";
 						}
 
 						++verified_files;
@@ -2189,7 +2252,7 @@ bool FileBackup::startFileMetadataDownloadThread()
 		std::auto_ptr<FileClient> fc_metadata_stream(new FileClient(false, identity, client_main->getProtocolVersions().filesrv_protocol_version,
 			client_main->isOnInternetConnection(), client_main, use_tmpfiles?NULL:this));
 
-		_u32 rc=client_main->getClientFilesrvConnection(fc_metadata_stream.get(), server_settings.get(), 10000);
+		_u32 rc=client_main->getClientFilesrvConnection(fc_metadata_stream.get(), server_settings.get(), 60000);
 		if(rc!=ERR_CONNECTED)
 		{
 			ServerLogger::Log(logid, "Backup of "+clientname+" failed - CONNECT error (for metadata stream)", LL_ERROR);
@@ -2392,7 +2455,7 @@ bool FileBackup::startPhashDownloadThread(const std::string& async_id)
 	std::auto_ptr<FileClient> fc_phash_stream(new FileClient(false, identity, client_main->getProtocolVersions().filesrv_protocol_version,
 		client_main->isOnInternetConnection(), client_main, use_tmpfiles ? NULL : this));
 
-	_u32 rc = client_main->getClientFilesrvConnection(fc_phash_stream.get(), server_settings.get(), 10000);
+	_u32 rc = client_main->getClientFilesrvConnection(fc_phash_stream.get(), server_settings.get(), 60000);
 	if (rc != ERR_CONNECTED)
 	{
 		ServerLogger::Log(logid, "Full Backup of " + clientname + " failed - CONNECT error (for metadata stream)", LL_ERROR);
