@@ -155,6 +155,35 @@ ServerBackupDao::CondInt ServerBackupDao::getClientGroup(int clientid)
 	return ret;
 }
 
+/**
+* @-SQLGenAccess
+* @func SSetting ServerBackupDao::getServerSetting
+* @return string value, string value_client, int use, int64 use_last_modified
+* @sql
+*		SELECT value, value_client, use, use_last_modified FROM settings_db.settings WHERE key=:key(string) AND clientid=:clientid(int)
+*/
+ServerBackupDao::SSetting ServerBackupDao::getServerSetting(const std::string& key, int clientid)
+{
+	if(q_getServerSetting==NULL)
+	{
+		q_getServerSetting=db->Prepare("SELECT value, value_client, use, use_last_modified FROM settings_db.settings WHERE key=? AND clientid=?", false);
+	}
+	q_getServerSetting->Bind(key);
+	q_getServerSetting->Bind(clientid);
+	db_results res=q_getServerSetting->Read();
+	q_getServerSetting->Reset();
+	SSetting ret = { false, "", "", 0, 0 };
+	if(!res.empty())
+	{
+		ret.exists=true;
+		ret.value=res[0]["value"];
+		ret.value_client=res[0]["value_client"];
+		ret.use=watoi(res[0]["use"]);
+		ret.use_last_modified=watoi64(res[0]["use_last_modified"]);
+	}
+	return ret;
+}
+
 
 /**
 * @-SQLGenAccess
@@ -182,49 +211,6 @@ ServerBackupDao::SClientName ServerBackupDao::getVirtualMainClientname(int clien
 	return ret;
 }
 
-/**
-* @-SQLGenAccess
-* @func void ServerBackupDao::insertIntoOrigClientSettings
-* @sql
-*      INSERT OR REPLACE INTO orig_client_settings (clientid, data)
-*          VALUES (:clientid(int), :data(std::string))
-*/
-void ServerBackupDao::insertIntoOrigClientSettings(int clientid, const std::string& data)
-{
-	if(q_insertIntoOrigClientSettings==NULL)
-	{
-		q_insertIntoOrigClientSettings=db->Prepare("INSERT OR REPLACE INTO orig_client_settings (clientid, data) VALUES (?, ?)", false);
-	}
-	q_insertIntoOrigClientSettings->Bind(clientid);
-	q_insertIntoOrigClientSettings->Bind(data);
-	q_insertIntoOrigClientSettings->Write();
-	q_insertIntoOrigClientSettings->Reset();
-}
-
-/**
-* @-SQLGenAccess
-* @func string ServerBackupDao::getOrigClientSettings
-* @return string data
-* @sql
-*      SELECT data FROM orig_client_settings WHERE clientid = :clientid(int)
-*/
-ServerBackupDao::CondString ServerBackupDao::getOrigClientSettings(int clientid)
-{
-	if(q_getOrigClientSettings==NULL)
-	{
-		q_getOrigClientSettings=db->Prepare("SELECT data FROM orig_client_settings WHERE clientid = ?", false);
-	}
-	q_getOrigClientSettings->Bind(clientid);
-	db_results res=q_getOrigClientSettings->Read();
-	q_getOrigClientSettings->Reset();
-	CondString ret = { false, "" };
-	if(!res.empty())
-	{
-		ret.exists=true;
-		ret.value=res[0]["data"];
-	}
-	return ret;
-}
 
 /**
 * @-SQLGenAccess
@@ -360,6 +346,31 @@ std::vector<int> ServerBackupDao::getClientsByUid(const std::string& uid)
 	return ret;
 }
 
+/**
+* @-SQLGenAccess
+* @func string ServerBackupDao::getClientUid
+* @return string uid
+* @sql
+*      SELECT uid FROM clients WHERE id=:id(int)
+*/
+ServerBackupDao::CondString ServerBackupDao::getClientUid(int id)
+{
+	if(q_getClientUid==NULL)
+	{
+		q_getClientUid=db->Prepare("SELECT uid FROM clients WHERE id=?", false);
+	}
+	q_getClientUid->Bind(id);
+	db_results res=q_getClientUid->Read();
+	q_getClientUid->Reset();
+	CondString ret = { false, "" };
+	if(!res.empty())
+	{
+		ret.exists=true;
+		ret.value=res[0]["uid"];
+	}
+	return ret;
+}
+
 /**
 * @-SQLGenAccess
 * @func void ServerBackupDao::updateClientUid
@@ -551,16 +562,19 @@ int ServerBackupDao::hasFileBackups(int clientid)
 * @-SQLGenAccess
 * @func void ServerBackupDao::insertSetting
 * @sql
-*      INSERT INTO settings_db.settings (key, value, clientid) VALUES ( :key(string), :value(string), :clientid(int) )
+*      INSERT INTO settings_db.settings (key, value, value_client, use, use_last_modified, clientid) VALUES ( :key(string), :value(string), :value_client(string), :use(int), :use_lm(int64), :clientid(int))
 */
-void ServerBackupDao::insertSetting(const std::string& key, const std::string& value, int clientid)
+void ServerBackupDao::insertSetting(const std::string& key, const std::string& value, const std::string& value_client, int use, int64 use_lm, int clientid)
 {
 	if(q_insertSetting==NULL)
 	{
-		q_insertSetting=db->Prepare("INSERT INTO settings_db.settings (key, value, clientid) VALUES ( ?, ?, ? )", false);
+		q_insertSetting=db->Prepare("INSERT INTO settings_db.settings (key, value, value_client, use, use_last_modified, clientid) VALUES ( ?, ?, ?, ?, ?, ?)", false);
 	}
 	q_insertSetting->Bind(key);
 	q_insertSetting->Bind(value);
+	q_insertSetting->Bind(value_client);
+	q_insertSetting->Bind(use);
+	q_insertSetting->Bind(use_lm);
 	q_insertSetting->Bind(clientid);
 	q_insertSetting->Write();
 	q_insertSetting->Reset();
@@ -585,6 +599,28 @@ void ServerBackupDao::updateSetting(const std::string& value, const std::string&
 	q_updateSetting->Reset();
 }
 
+/**
+* @-SQLGenAccess
+* @func void ServerBackupDao::updateSettingAll
+* @sql
+*      UPDATE settings_db.settings SET value=:value(string), value_client=:value_client(string, use=:use(int), use_last_modified=:use_lm(int64) WHERE key=:key(string) AND clientid=:clientid(int)
+*/
+void ServerBackupDao::updateSettingAll(const std::string& value, const std::string& value_client, int use, int64 use_lm, const std::string& key, int clientid)
+{
+	if(q_updateSettingAll==NULL)
+	{
+		q_updateSettingAll=db->Prepare("UPDATE settings_db.settings SET value=?, value_client=?, use=?, use_last_modified=? WHERE key=? AND clientid=?", false);
+	}
+	q_updateSettingAll->Bind(value);
+	q_updateSettingAll->Bind(value_client);
+	q_updateSettingAll->Bind(use);
+	q_updateSettingAll->Bind(use_lm);
+	q_updateSettingAll->Bind(key);
+	q_updateSettingAll->Bind(clientid);
+	q_updateSettingAll->Write();
+	q_updateSettingAll->Reset();
+}
+
 /**
 * @-SQLGenAccess
 * @func string ServerBackupDao::getMiscValue
@@ -1231,18 +1267,19 @@ void ServerBackupDao::updateClientLastFileBackup(int backupid, int last_fileback
 * @-SQLGenAccess
 * @func void ServerBackupDao::updateClientOsAndClientVersion
 * @sql
-*       UPDATE clients SET os_simple=:os_simple(string), os_version_str=:os_version(string), client_version_str=:client_version(string)
+*       UPDATE clients SET os_simple=:os_simple(string), os_version_str=:os_version(string), client_version_str=:client_version(string), capa=:capa(int)
 *		WHERE id=:clientid(int)
 */
-void ServerBackupDao::updateClientOsAndClientVersion(const std::string& os_simple, const std::string& os_version, const std::string& client_version, int clientid)
+void ServerBackupDao::updateClientOsAndClientVersion(const std::string& os_simple, const std::string& os_version, const std::string& client_version, int capa, int clientid)
 {
 	if(q_updateClientOsAndClientVersion==NULL)
 	{
-		q_updateClientOsAndClientVersion=db->Prepare("UPDATE clients SET os_simple=?, os_version_str=?, client_version_str=? WHERE id=?", false);
+		q_updateClientOsAndClientVersion=db->Prepare("UPDATE clients SET os_simple=?, os_version_str=?, client_version_str=?, capa=? WHERE id=?", false);
 	}
 	q_updateClientOsAndClientVersion->Bind(os_simple);
 	q_updateClientOsAndClientVersion->Bind(os_version);
 	q_updateClientOsAndClientVersion->Bind(client_version);
+	q_updateClientOsAndClientVersion->Bind(capa);
 	q_updateClientOsAndClientVersion->Bind(clientid);
 	q_updateClientOsAndClientVersion->Write();
 	q_updateClientOsAndClientVersion->Reset();
@@ -1922,6 +1959,93 @@ std::vector<ServerBackupDao::SMountedImage> ServerBackupDao::getOldMountedImages
 	return ret;
 }
 
+/**
+* @-SQLGenAccess
+* @func void ServerBackupDao::setCapa
+* @sql
+*       UPDATE clients SET capa=:capa(int) WHERE id=:clientid(int)
+*/
+void ServerBackupDao::setCapa(int capa, int clientid)
+{
+	if(q_setCapa==NULL)
+	{
+		q_setCapa=db->Prepare("UPDATE clients SET capa=? WHERE id=?", false);
+	}
+	q_setCapa->Bind(capa);
+	q_setCapa->Bind(clientid);
+	q_setCapa->Write();
+	q_setCapa->Reset();
+}
+
+/**
+* @-SQLGenAccess
+* @func int ServerBackupDao::getCapa
+* @return int capa
+* @sql
+*       SELECT capa FROM clients WHERE id=:clientid(int)
+*/
+ServerBackupDao::CondInt ServerBackupDao::getCapa(int clientid)
+{
+	if(q_getCapa==NULL)
+	{
+		q_getCapa=db->Prepare("SELECT capa FROM clients WHERE id=?", false);
+	}
+	q_getCapa->Bind(clientid);
+	db_results res=q_getCapa->Read();
+	q_getCapa->Reset();
+	CondInt ret = { false, 0 };
+	if(!res.empty())
+	{
+		ret.exists=true;
+		ret.value=watoi(res[0]["capa"]);
+	}
+	return ret;
+}
+
+/**
+* @-SQLGenAccess
+* @func int ServerBackupDao::getClientWithHashes
+* @return int with_hashes
+* @sql
+*       SELECT with_hashes FROM clients WHERE id=:clientid(int)
+*/
+ServerBackupDao::CondInt ServerBackupDao::getClientWithHashes(int clientid)
+{
+	if(q_getClientWithHashes==NULL)
+	{
+		q_getClientWithHashes=db->Prepare("SELECT with_hashes FROM clients WHERE id=?", false);
+	}
+	q_getClientWithHashes->Bind(clientid);
+	db_results res=q_getClientWithHashes->Read();
+	q_getClientWithHashes->Reset();
+	CondInt ret = { false, 0 };
+	if(!res.empty())
+	{
+		ret.exists=true;
+		ret.value=watoi(res[0]["with_hashes"]);
+	}
+	return ret;
+}
+
+/**
+* @-SQLGenAccess
+* @func void ServerBackupDao::updateClientWithHashes
+* @sql
+*		UPDATE clients SET with_hashes=:with_hashes(int) WHERE id=:clientid(int)
+*/
+void ServerBackupDao::updateClientWithHashes(int with_hashes, int clientid)
+{
+	if(q_updateClientWithHashes==NULL)
+	{
+		q_updateClientWithHashes=db->Prepare("UPDATE clients SET with_hashes=? WHERE id=?", false);
+	}
+	q_updateClientWithHashes->Bind(with_hashes);
+	q_updateClientWithHashes->Bind(clientid);
+	q_updateClientWithHashes->Write();
+	q_updateClientWithHashes->Reset();
+}
+
+
 //@-SQLGenSetup
 void ServerBackupDao::prepareQueries( void )
 {
@@ -1930,14 +2054,14 @@ void ServerBackupDao::prepareQueries( void )
 	q_getDeletePendingClientNames=NULL;
 	q_getGroupName=NULL;
 	q_getClientGroup=NULL;
+	q_getServerSetting=NULL;
 	q_getVirtualMainClientname=NULL;
-	q_insertIntoOrigClientSettings=NULL;
-	q_getOrigClientSettings=NULL;
 	q_getLastIncrementalDurations=NULL;
 	q_getLastFullDurations=NULL;
 	q_getClientSetting=NULL;
 	q_getClientIds=NULL;
 	q_getClientsByUid=NULL;
+	q_getClientUid=NULL;
 	q_updateClientUid=NULL;
 	q_deleteClient=NULL;
 	q_changeClientName=NULL;
@@ -1949,6 +2073,7 @@ void ServerBackupDao::prepareQueries( void )
 	q_hasFileBackups=NULL;
 	q_insertSetting=NULL;
 	q_updateSetting=NULL;
+	q_updateSettingAll=NULL;
 	q_getMiscValue=NULL;
 	q_addMiscValue=NULL;
 	q_delMiscValue=NULL;
@@ -2008,6 +2133,10 @@ void ServerBackupDao::prepareQueries( void )
 	q_getMountedImage=NULL;
 	q_getImageInfo=NULL;
 	q_getOldMountedImages=NULL;
+	q_setCapa=NULL;
+	q_getCapa=NULL;
+	q_getClientWithHashes=NULL;
+	q_updateClientWithHashes=NULL;
 }
 
 //@-SQLGenDestruction
@@ -2018,14 +2147,14 @@ void ServerBackupDao::destroyQueries( void )
 	db->destroyQuery(q_getDeletePendingClientNames);
 	db->destroyQuery(q_getGroupName);
 	db->destroyQuery(q_getClientGroup);
+	db->destroyQuery(q_getServerSetting);
 	db->destroyQuery(q_getVirtualMainClientname);
-	db->destroyQuery(q_insertIntoOrigClientSettings);
-	db->destroyQuery(q_getOrigClientSettings);
 	db->destroyQuery(q_getLastIncrementalDurations);
 	db->destroyQuery(q_getLastFullDurations);
 	db->destroyQuery(q_getClientSetting);
 	db->destroyQuery(q_getClientIds);
 	db->destroyQuery(q_getClientsByUid);
+	db->destroyQuery(q_getClientUid);
 	db->destroyQuery(q_updateClientUid);
 	db->destroyQuery(q_deleteClient);
 	db->destroyQuery(q_changeClientName);
@@ -2037,6 +2166,7 @@ void ServerBackupDao::destroyQueries( void )
 	db->destroyQuery(q_hasFileBackups);
 	db->destroyQuery(q_insertSetting);
 	db->destroyQuery(q_updateSetting);
+	db->destroyQuery(q_updateSettingAll);
 	db->destroyQuery(q_getMiscValue);
 	db->destroyQuery(q_addMiscValue);
 	db->destroyQuery(q_delMiscValue);
@@ -2096,17 +2226,21 @@ void ServerBackupDao::destroyQueries( void )
 	db->destroyQuery(q_getMountedImage);
 	db->destroyQuery(q_getImageInfo);
 	db->destroyQuery(q_getOldMountedImages);
+	db->destroyQuery(q_setCapa);
+	db->destroyQuery(q_getCapa);
+	db->destroyQuery(q_getClientWithHashes);
+	db->destroyQuery(q_updateClientWithHashes);
 }
 
 
-void ServerBackupDao::updateOrInsertSetting( int clientid, const std::string& key, const std::string& value )
+void ServerBackupDao::updateOrInsertSetting( int clientid, const std::string& key, const std::string& value, const std::string& value_client, int use, int64 use_lm)
 {
 	if(getSetting(clientid, key).exists)
 	{
-		updateSetting(value, key, clientid);
+		updateSettingAll(value, value_client, use, use_lm, key, clientid);
 	}
 	else
 	{
-		insertSetting(key, value, clientid);
+		insertSetting(key, value, value_client, use, use_lm, clientid);
 	}
 }
