@@ -4,6 +4,7 @@
 #include "../Interface/SettingsReader.h"
 #include "../Interface/Database.h"
 #include "../Interface/Mutex.h"
+#include "../urbackupcommon/json.h"
 #include <memory>
 
 namespace
@@ -12,12 +13,20 @@ namespace
 	const char* image_file_format_vhd = "vhd";
 	const char* image_file_format_vhdz = "vhdz";
 	const char* image_file_format_cowraw = "cowraw";
+	const char* image_file_format_vhdx = "vhdx";
+	const char* image_file_format_vhdxz = "vhdxz";
 
 	const char* full_image_style_full = "full";
 	const char* full_image_style_synthetic = "synthetic";
 
 	const char* incr_image_style_to_full = "to-full";
 	const char* incr_image_style_to_last = "to-last";
+
+	const int c_use_undefined = -1;
+	const int c_use_group = 1;
+	const int c_use_value = 2;
+	const char* c_use_value_str = "2";
+	const int c_use_value_client = 4;
 }
 
 struct SSettings
@@ -46,7 +55,6 @@ struct SSettings
 	int max_image_full;
 	bool no_images;
 	bool no_file_backups;
-	bool overwrite;
 	bool allow_overwrite;
 	bool autoshutdown;
 	int startup_backup_delay;
@@ -78,7 +86,7 @@ struct SSettings
 	bool backup_database;
 	std::string internet_server;
 	bool client_set_settings;
-	unsigned short internet_server_port;
+	std::string internet_server_port;
 	std::string internet_server_proxy;
 	std::string internet_authkey;
 	bool internet_full_file_backups;
@@ -137,6 +145,14 @@ struct SSettings
 	int64 internet_image_dataplan_limit;
 	int alert_script;
 	std::string alert_params;
+	std::string archive;
+	std::string client_settings_tray_access_pw;
+	bool local_encrypt;
+	bool local_compress;
+	int download_threads;
+	int hash_threads;
+	int client_hash_threads;
+	int image_compress_threads;
 };
 
 struct SLDAPSettings
@@ -236,15 +252,32 @@ public:
 
 	std::string ldapMapToString(const std::map<std::string, std::string>& ldap_map);
 
-	void createSettingsReaders(std::auto_ptr<ISettingsReader>& settings_default,
-		std::auto_ptr<ISettingsReader>& settings_client, std::auto_ptr<ISettingsReader>& settings_global);
+	static void createSettingsReaders(IDatabase* db, int clientid, std::auto_ptr<ISettingsReader>& settings_default,
+		std::auto_ptr<ISettingsReader>& settings_client, std::auto_ptr<ISettingsReader>& settings_global, int& settings_default_id);
+
+	std::string getImageFileFormatInt(const std::string& image_file_format);
+
+	static void readStringClientSetting(IDatabase* db, int clientid, const std::string &name, const std::string& merge_sep, std::string *output, bool allow_client_value = true);
+
+
+	struct SClientSetting
+	{
+		SClientSetting()
+			: use(-1)
+		{}
+
+		JSON::Value value;
+		JSON::Value value_client;
+		JSON::Value value_group;
+		int use;
+	};
+
+	static std::map<std::string, SClientSetting> getClientSettings(IDatabase* db, int t_clientid, int group_id = 0, IDatabase* group_db = NULL);
 
 private:
 	void operator=(const ServerSettings& other){};
 	ServerSettings(const ServerSettings& other){};
 
-	std::string getImageFileFormatInt(const std::string& image_file_format);
-
 	std::vector<STimeSpan> getWindow(std::string window);
 
 	std::vector<STimeSpan> parseTimeSpan(std::string time_span);
@@ -253,17 +286,24 @@ private:
 
 	double currentTimeSpanValue(std::string time_span_value);
 
+	void readSettings();
+
 
 	float parseTimeDet(std::string t);
 	STimeSpan parseTime(std::string t);
 	int parseDayOfWeek(std::string dow);
-	void readSettingsDefault(ISettingsReader* settings_default, ISettingsReader* settings_global);
-	void readSettingsClient(ISettingsReader* settings_client);
-	void readBoolClientSetting(ISettingsReader* settings_client, const std::string &name, bool *output);
-	void readStringClientSetting(ISettingsReader* settings_client, const std::string &name, std::string *output);
-	void readIntClientSetting(ISettingsReader* settings_client, const std::string &name, int *output);
-	void readInt64ClientSetting(ISettingsReader* settings_client, const std::string &name, int64 *output);
-	void readSizeClientSetting(ISettingsReader* settings_client, const std::string &name, size_t *output);
+	void readSettingsDefault(ISettingsReader* settings_default, ISettingsReader* settings_global, IQuery* q_get_client_setting);
+	void readSettingsClient(ISettingsReader* settings_client, IQuery* q_get_client_setting);
+
+	void readStringClientSetting(IQuery* q_get_client_setting, const std::string &name, const std::string& merge_sep, std::string *output, bool allow_client_value=true);
+	static void readStringClientSetting(IQuery* q_get_client_setting, int clientid, const std::string &name, const std::string& merge_sep, std::string *output, bool allow_client_value = true);
+	std::string readValClientSetting(IQuery* q_get_client_setting, const std::string &name, bool allow_client_value);
+	void readBoolClientSetting(IQuery* q_get_client_setting, const std::string &name, bool *output, bool allow_client_value = true);
+	void readIntClientSetting(IQuery* q_get_client_setting, const std::string &name, int *output, bool allow_client_value = true);
+	void readInt64ClientSetting(IQuery* q_get_client_setting, const std::string &name, int64 *output, bool allow_client_value = true);
+	void readSizeClientSetting(IQuery* q_get_client_setting, const std::string &name, size_t *output, bool allow_client_value = true);
+	void readFloatClientSetting(IQuery* q_get_client_setting, const std::string& name, float* output, bool allow_client_value = true);
+
 	void updateInternal(bool* was_updated);
 	std::map<std::string, std::string> parseLdapMap(const std::string& data);
 
